import{s as Gt,n as Rt,o as Bt}from"../chunks/scheduler.c57aa7ef.js";import{S as Kt,i as Qt,g as r,s as i,r as f,A as Jt,h as s,f as e,c as o,j as P,u,x as m,k as w,y as a,a as l,v as h,d as b,t as v,w as _}from"../chunks/index.394f8801.js";import{D as W}from"../chunks/Docstring.46378b0f.js";import{H as Et,E as Wt}from"../chunks/EditOnGithub.157fddbf.js";function Xt(Ot){let x,Z,X,tt,k,et,L,Mt='<a href="https://huggingface.co/papers/2410.17222" rel="nofollow">CPT</a> combines In-Context Learning (ICL), Prompt Tuning (PT), and adversarial optimization to improve few-shot learning by refining context embeddings. CPT updates the context tokens by optimizing both the context and the training examples, encapsulating them into a novel loss design that minimizes overfitting, enables more effective optimization, and drives significant improvements in classification tasks.',nt,E,It="The abstract from the paper is:",at,O,Nt="<p>Large Language Models (LLMs) can perform few-shot learning using either optimization-based approaches or In-Context Learning (ICL). Optimization-based methods often suffer from overfitting, as they require updating a large number of parameters with limited data. In contrast, ICL avoids overfitting but typically underperforms compared to optimization-based methods and is highly sensitive to the selection, order, and format of demonstration examples. To overcome these challenges, we introduce Context-aware Prompt Tuning (CPT), a method inspired by ICL, Prompt Tuning (PT), and adversarial attacks. CPT builds on the ICL strategy of concatenating examples before the input, extending it by incorporating PT-like learning to refine the context embedding through iterative optimization, extracting deeper insights from the training examples. Our approach carefully modifies specific context tokens, considering the unique structure of the examples within the context. In addition to updating the context with PT-like optimization, CPT draws inspiration from adversarial attacks, adjusting the input based on the labels present in the context while preserving the inherent value of the user-provided data. To ensure robustness and stability during optimization, we employ a projected gradient descent algorithm, constraining token embeddings to remain close to their original values and safeguarding the quality of the context. Our method has demonstrated superior accuracy across multiple classification tasks using various LLM models, outperforming existing baselines and effectively addressing the overfitting challenge in few-shot learning.</p>",it,M,zt='Take a look at <a href="https://github.com/huggingface/peft/blob/main/examples/cpt_finetuning/README.md" rel="nofollow">Example</a> for a step-by-step guide on how to train a model with CPT.',ot,I,rt,d,N,ht,S,jt="CPT Configuration class extending PeftConfig for Context-aware Prompt Tuning (CPT).",bt,U,Dt="This class introduces additional parameters required for CPT, such as:",vt,F,Ht="<li>Token type masks</li> <li>Prompt tuning initialization</li> <li>Loss weighting</li> <li>Projection settings</li>",_t,G,qt='For more details, see the paper: <a href="https://arxiv.org/abs/2410.17222" rel="nofollow">https://arxiv.org/abs/2410.17222</a>',st,z,lt,p,j,xt,R,At=`CPTEmbedding is a custom embedding layer designed for Context-aware Prompt Tuning (CPT) in PEFT. It initializes
embeddings, applies prompt-specific projections, and computes loss using label masks.`,Ct,C,D,$t,B,Vt="Computes the loss for CPT models with optional exponential decay.",Tt,$,H,yt,K,St="Computes the prompt embeddings and applies delta adjustments.",Pt,T,q,wt,Q,Ut="Applies epsilon-based projection to the delta embeddings to control their norm.",kt,y,A,Lt,J,Ft="Sets up a backward hook to selectively update token gradients based on the CPT token type mask.",pt,V,dt,Y,mt;return k=new Et({props:{title:"Context-aware Prompt Tuning: Advancing In-Context Learning with Adversarial Methods",local:"context-aware-prompt-tuning-advancing-in-context-learning-with-adversarial-methods",headingTag:"h1"}}),I=new Et({props:{title:"CPTConfig",local:"peft.CPTConfig",headingTag:"h2"}}),N=new W({props:{name:"class peft.CPTConfig",anchor:"peft.CPTConfig",parameters:[{name:"task_type",val:": typing.Union[str, peft.utils.peft_types.TaskType, NoneType] = None"},{name:"peft_type",val:": typing.Union[str, peft.utils.peft_types.PeftType, NoneType] = None"},{name:"auto_mapping",val:": typing.Optional[dict] = None"},{name:"base_model_name_or_path",val:": typing.Optional[str] = None"},{name:"revision",val:": typing.Optional[str] = None"},{name:"inference_mode",val:": bool = False"},{name:"num_virtual_tokens",val:": int = None"},{name:"token_dim",val:": int = None"},{name:"num_transformer_submodules",val:": typing.Optional[int] = None"},{name:"num_attention_heads",val:": typing.Optional[int] = None"},{name:"num_layers",val:": typing.Optional[int] = None"},{name:"cpt_token_ids",val:": typing.Optional[list[int]] = None"},{name:"cpt_mask",val:": typing.Optional[list[int]] = None"},{name:"cpt_tokens_type_mask",val:": typing.Optional[list[int]] = None"},{name:"opt_weighted_loss_type",val:": typing.Optional[typing.Literal['none', 'decay']] = 'none'"},{name:"opt_loss_decay_factor",val:": typing.Optional[float] = 1.0"},{name:"opt_projection_epsilon",val:": typing.Optional[float] = 0.1"},{name:"opt_projection_format_epsilon",val:": typing.Optional[float] = 0.1"},{name:"tokenizer_name_or_path",val:": typing.Optional[str] = None"}],source:"https://github.com/huggingface/peft/blob/main/src/peft/tuners/cpt/config.py#L22"}}),z=new Et({props:{title:"CPTEmbedding",local:"peft.CPTEmbedding",headingTag:"h2"}}),j=new W({props:{name:"class peft.CPTEmbedding",anchor:"peft.CPTEmbedding",parameters:[{name:"config",val:""},{name:"word_embeddings",val:""}],source:"https://github.com/huggingface/peft/blob/main/src/peft/tuners/cpt/model.py#L23"}}),D=new W({props:{name:"calculate_loss",anchor:"peft.CPTEmbedding.calculate_loss",parameters:[{name:"base_model_output",val:""},{name:"labels",val:""},{name:"cpt_type_mask",val:""},{name:"config",val:""}],parametersDescription:[{anchor:"peft.CPTEmbedding.calculate_loss.base_model_output",description:`<strong>base_model_output</strong> (ModelOutput) &#x2014;
Output from the base model containing logits.`,name:"base_model_output"},{anchor:"peft.CPTEmbedding.calculate_loss.labels",description:`<strong>labels</strong> (torch.Tensor) &#x2014;
Ground-truth labels for the input tokens.`,name:"labels"},{anchor:"peft.CPTEmbedding.calculate_loss.cpt_type_mask",description:`<strong>cpt_type_mask</strong> (torch.Tensor) &#x2014;
Token type mask used for filtering valid loss terms.`,name:"cpt_type_mask"},{anchor:"peft.CPTEmbedding.calculate_loss.config",description:`<strong>config</strong> (Namespace) &#x2014;
Configuration object containing loss-related hyperparameters.`,name:"config"}],source:"https://github.com/huggingface/peft/blob/main/src/peft/tuners/cpt/model.py#L141",returnDescription:`<script context="module">export const metadata = 'undefined';<\/script>


<p>The base model output with computed loss.</p>
`,returnType:`<script context="module">export const metadata = 'undefined';<\/script>


<p>ModelOutput</p>
`}}),H=new W({props:{name:"forward",anchor:"peft.CPTEmbedding.forward",parameters:[{name:"indices",val:""}],parametersDescription:[{anchor:"peft.CPTEmbedding.forward.indices",description:`<strong>indices</strong> (torch.Tensor) &#x2014;
Indices of the tokens to be embedded.`,name:"indices"}],source:"https://github.com/huggingface/peft/blob/main/src/peft/tuners/cpt/model.py#L63",returnDescription:`<script context="module">export const metadata = 'undefined';<\/script>


<p>Sum of prompt embeddings and delta embeddings.</p>
`,returnType:`<script context="module">export const metadata = 'undefined';<\/script>


<p>torch.Tensor</p>
`}}),q=new W({props:{name:"get_projection",anchor:"peft.CPTEmbedding.get_projection",parameters:[],source:"https://github.com/huggingface/peft/blob/main/src/peft/tuners/cpt/model.py#L123"}}),A=new W({props:{name:"set_updated_tokens",anchor:"peft.CPTEmbedding.set_updated_tokens",parameters:[],source:"https://github.com/huggingface/peft/blob/main/src/peft/tuners/cpt/model.py#L84"}}),V=new Wt({props:{source:"https://github.com/huggingface/peft/blob/main/docs/source/package_reference/cpt.md"}}),{c(){x=r("meta"),Z=i(),X=r("p"),tt=i(),f(k.$$.fragment),et=i(),L=r("p"),L.innerHTML=Mt,nt=i(),E=r("p"),E.textContent=It,at=i(),O=r("blockquote"),O.innerHTML=Nt,it=i(),M=r("p"),M.innerHTML=zt,ot=i(),f(I.$$.fragment),rt=i(),d=r("div"),f(N.$$.fragment),ht=i(),S=r("p"),S.textContent=jt,bt=i(),U=r("p"),U.textContent=Dt,vt=i(),F=r("ul"),F.innerHTML=Ht,_t=i(),G=r("p"),G.innerHTML=qt,st=i(),f(z.$$.fragment),lt=i(),p=r("div"),f(j.$$.fragment),xt=i(),R=r("p"),R.textContent=At,Ct=i(),C=r("div"),f(D.$$.fragment),$t=i(),B=r("p"),B.textContent=Vt,Tt=i(),$=r("div"),f(H.$$.fragment),yt=i(),K=r("p"),K.textContent=St,Pt=i(),T=r("div"),f(q.$$.fragment),wt=i(),Q=r("p"),Q.textContent=Ut,kt=i(),y=r("div"),f(A.$$.fragment),Lt=i(),J=r("p"),J.textContent=Ft,pt=i(),f(V.$$.fragment),dt=i(),Y=r("p"),this.h()},l(t){const n=Jt("svelte-u9bgzb",document.head);x=s(n,"META",{name:!0,content:!0}),n.forEach(e),Z=o(t),X=s(t,"P",{}),P(X).forEach(e),tt=o(t),u(k.$$.fragment,t),et=o(t),L=s(t,"P",{"data-svelte-h":!0}),m(L)!=="svelte-nhd3j4"&&(L.innerHTML=Mt),nt=o(t),E=s(t,"P",{"data-svelte-h":!0}),m(E)!=="svelte-1cwsb16"&&(E.textContent=It),at=o(t),O=s(t,"BLOCKQUOTE",{"data-svelte-h":!0}),m(O)!=="svelte-166ut3c"&&(O.innerHTML=Nt),it=o(t),M=s(t,"P",{"data-svelte-h":!0}),m(M)!=="svelte-ih8qk1"&&(M.innerHTML=zt),ot=o(t),u(I.$$.fragment,t),rt=o(t),d=s(t,"DIV",{class:!0});var g=P(d);u(N.$$.fragment,g),ht=o(g),S=s(g,"P",{"data-svelte-h":!0}),m(S)!=="svelte-pvpmqk"&&(S.textContent=jt),bt=o(g),U=s(g,"P",{"data-svelte-h":!0}),m(U)!=="svelte-jwcs2b"&&(U.textContent=Dt),vt=o(g),F=s(g,"UL",{"data-svelte-h":!0}),m(F)!=="svelte-aqudvo"&&(F.innerHTML=Ht),_t=o(g),G=s(g,"P",{"data-svelte-h":!0}),m(G)!=="svelte-1hxqr0v"&&(G.innerHTML=qt),g.forEach(e),st=o(t),u(z.$$.fragment,t),lt=o(t),p=s(t,"DIV",{class:!0});var c=P(p);u(j.$$.fragment,c),xt=o(c),R=s(c,"P",{"data-svelte-h":!0}),m(R)!=="svelte-1cdhmyj"&&(R.textContent=At),Ct=o(c),C=s(c,"DIV",{class:!0});var ct=P(C);u(D.$$.fragment,ct),$t=o(ct),B=s(ct,"P",{"data-svelte-h":!0}),m(B)!=="svelte-9bizft"&&(B.textContent=Vt),ct.forEach(e),Tt=o(c),$=s(c,"DIV",{class:!0});var gt=P($);u(H.$$.fragment,gt),yt=o(gt),K=s(gt,"P",{"data-svelte-h":!0}),m(K)!=="svelte-3v5t0c"&&(K.textContent=St),gt.forEach(e),Pt=o(c),T=s(c,"DIV",{class:!0});var ft=P(T);u(q.$$.fragment,ft),wt=o(ft),Q=s(ft,"P",{"data-svelte-h":!0}),m(Q)!=="svelte-pg5azd"&&(Q.textContent=Ut),ft.forEach(e),kt=o(c),y=s(c,"DIV",{class:!0});var ut=P(y);u(A.$$.fragment,ut),Lt=o(ut),J=s(ut,"P",{"data-svelte-h":!0}),m(J)!=="svelte-1bcpx1v"&&(J.textContent=Ft),ut.forEach(e),c.forEach(e),pt=o(t),u(V.$$.fragment,t),dt=o(t),Y=s(t,"P",{}),P(Y).forEach(e),this.h()},h(){w(x,"name","hf:doc:metadata"),w(x,"content",Yt),w(d,"class","docstring border-l-2 border-t-2 pl-4 pt-3.5 border-gray-100 rounded-tl-xl mb-6 mt-8"),w(C,"class","docstring border-l-2 border-t-2 pl-4 pt-3.5 border-gray-100 rounded-tl-xl mb-6 mt-8"),w($,"class","docstring border-l-2 border-t-2 pl-4 pt-3.5 border-gray-100 rounded-tl-xl mb-6 mt-8"),w(T,"class","docstring border-l-2 border-t-2 pl-4 pt-3.5 border-gray-100 rounded-tl-xl mb-6 mt-8"),w(y,"class","docstring border-l-2 border-t-2 pl-4 pt-3.5 border-gray-100 rounded-tl-xl mb-6 mt-8"),w(p,"class","docstring border-l-2 border-t-2 pl-4 pt-3.5 border-gray-100 rounded-tl-xl mb-6 mt-8")},m(t,n){a(document.head,x),l(t,Z,n),l(t,X,n),l(t,tt,n),h(k,t,n),l(t,et,n),l(t,L,n),l(t,nt,n),l(t,E,n),l(t,at,n),l(t,O,n),l(t,it,n),l(t,M,n),l(t,ot,n),h(I,t,n),l(t,rt,n),l(t,d,n),h(N,d,null),a(d,ht),a(d,S),a(d,bt),a(d,U),a(d,vt),a(d,F),a(d,_t),a(d,G),l(t,st,n),h(z,t,n),l(t,lt,n),l(t,p,n),h(j,p,null),a(p,xt),a(p,R),a(p,Ct),a(p,C),h(D,C,null),a(C,$t),a(C,B),a(p,Tt),a(p,$),h(H,$,null),a($,yt),a($,K),a(p,Pt),a(p,T),h(q,T,null),a(T,wt),a(T,Q),a(p,kt),a(p,y),h(A,y,null),a(y,Lt),a(y,J),l(t,pt,n),h(V,t,n),l(t,dt,n),l(t,Y,n),mt=!0},p:Rt,i(t){mt||(b(k.$$.fragment,t),b(I.$$.fragment,t),b(N.$$.fragment,t),b(z.$$.fragment,t),b(j.$$.fragment,t),b(D.$$.fragment,t),b(H.$$.fragment,t),b(q.$$.fragment,t),b(A.$$.fragment,t),b(V.$$.fragment,t),mt=!0)},o(t){v(k.$$.fragment,t),v(I.$$.fragment,t),v(N.$$.fragment,t),v(z.$$.fragment,t),v(j.$$.fragment,t),v(D.$$.fragment,t),v(H.$$.fragment,t),v(q.$$.fragment,t),v(A.$$.fragment,t),v(V.$$.fragment,t),mt=!1},d(t){t&&(e(Z),e(X),e(tt),e(et),e(L),e(nt),e(E),e(at),e(O),e(it),e(M),e(ot),e(rt),e(d),e(st),e(lt),e(p),e(pt),e(dt),e(Y)),e(x),_(k,t),_(I,t),_(N),_(z,t),_(j),_(D),_(H),_(q),_(A),_(V,t)}}}const Yt='{"title":"Context-aware Prompt Tuning: Advancing In-Context Learning with Adversarial Methods","local":"context-aware-prompt-tuning-advancing-in-context-learning-with-adversarial-methods","sections":[{"title":"CPTConfig","local":"peft.CPTConfig","sections":[],"depth":2},{"title":"CPTEmbedding","local":"peft.CPTEmbedding","sections":[],"depth":2}],"depth":1}';function Zt(Ot){return Bt(()=>{new URLSearchParams(window.location.search).get("fw")}),[]}class ie extends Kt{constructor(x){super(),Qt(this,x,Zt,Xt,Gt,{})}}export{ie as component};
